/*
 * This file is part of FGGL.
 *
 * FGGL is free software: you can redistribute it and/or modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation, either version 3 of the License, or (at your option) any
 * later version.
 *
 * FGGL is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; without even the implied warranty
 * of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public License along with FGGL.
 * If not, see <https://www.gnu.org/licenses/>.
 */

//
// Created by webpigeon on 12/07/23.
//

#ifndef FGGL_COMPONENTS_CORE_FILESYSTEM_READER_HPP
#define FGGL_COMPONENTS_CORE_FILESYSTEM_READER_HPP

#include <filesystem>
#include <optional>
#include <vector>

#include "fggl/services/service.hpp"
#include "fggl/services/module.hpp"
#include "fggl/platform/paths.hpp"
#include "Finder.hpp"

namespace fggl::filesystem {

	class Reader {
		public:
			FGGL_DEFINE_SERVICE(fggl::filesystem::Reader)

			using Path = std::filesystem::path;

			inline explicit Reader(Locator* locator) : m_locator(locator){};

			std::optional<std::string> readText(const char* name, Context context = Context::RESOURCE, Owner owner = Owner::ANY);
			char* readBytes(const char* name, Context context = Context::RESOURCE, Owner owner = Owner::ANY);

			template<typename T>
			std::optional<T> readAs(const char* name, std::function<std::optional<T>(Path&)> converter, Context context = Context::RESOURCE, Owner owner = Owner::ANY) {
				auto path = m_locator->find(name, context, owner);
				if ( !path.has_value() ){
					return {};
				}
				return converter(path);
			}

		private:
			Locator* m_locator;

	};

} // namespace fggl::filesystem

#endif //FGGL_COMPONENTS_CORE_FILESYSTEM_READER_HPP
