/*
 * This file is part of FGGL.
 *
 * FGGL is free software: you can redistribute it and/or modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation, either version 3 of the License, or (at your option) any
 * later version.
 *
 * FGGL is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; without even the implied warranty
 * of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public License along with FGGL.
 * If not, see <https://www.gnu.org/licenses/>.
 */

//
// Created by webpigeon on 12/07/23.
//

#include "fggl/filesystem/reader.hpp"

#include <fstream>

namespace fggl::filesystem {

	std::optional<std::string> Reader::readText(const char *name, Context context, Owner owner) {
		auto path = m_locator->find(name, context, owner);
		if ( !path.has_value() ) {
			return {};
		}

		std::string output;
		std::ifstream ifs(path.value());
		output.assign( std::istreambuf_iterator<char>(ifs), (std::istreambuf_iterator<char>()) );
		return output;
	}

	char* Reader::readBytes(const char *name, Context context, Owner owner) {
		auto path = m_locator->find(name, context, owner);
		if ( !path.has_value() ) {
			return {};
		}

		// get stream
		std::ifstream buff(path.value(), std::ios::in | std::ios::binary );

		// figure out length
		buff.seekg(0, std::ios::end);
		auto length = buff.tellg();

		// read whole file into buffer
		buff.seekg(0, std::ios::beg);
		char *data = new char[length];
		buff.read(data, length);

		return data;
	}

} // namespace fggl::filesystem