/*
 * This file is part of FGGL.
 *
 * FGGL is free software: you can redistribute it and/or modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation, either version 3 of the License, or (at your option) any
 * later version.
 *
 * FGGL is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; without even the implied warranty
 * of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public License along with FGGL.
 * If not, see <https://www.gnu.org/licenses/>.
 */

//
// Created by webpigeon on 10/07/23.
//

#include "fggl/services/factory.hpp"
#include <iostream>

namespace fggl::services {

	namespace {
		[[maybe_unused]]
		void dump_factories( const std::map<ServiceName, std::shared_ptr<ServiceBinding>>& factories ){
			std::cerr << "BEGIN registered factories" << std::endl;
			for ( auto& factory : factories ) {
				std::cerr << "\t" << factory.first.get() << std::endl;
			}
			std::cerr << "END registered factories" << std::endl;
		}
	}

	std::shared_ptr<void> Generator::invokeFactoryRaw(ServiceName name) {
		assert( m_factories.find(name) != m_factories.end() && "Missing service, either module order is wrong, or nothing provides this service" );

		// invoke and build
		auto& meta = m_factories.at(name);

		#ifndef NDEBUG
			auto provided = meta->provide();
			assert( provided != nullptr );
			return provided;
		#else
			return meta->provide();
		#endif
	}

} // namespace fggl::services
