/*
 * This file is part of FGGL.
 *
 * FGGL is free software: you can redistribute it and/or modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation, either version 3 of the License, or (at your option) any
 * later version.
 *
 * FGGL is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; without even the implied warranty
 * of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public License along with FGGL.
 * If not, see <https://www.gnu.org/licenses/>.
 */

//
// Created by webpigeon on 12/07/23.
//

#include "fggl/services/module.hpp"

namespace fggl::services {

	/**
	 * Report what contracts this module satisfies.
	 *
	 * By default, a module only provides its name. In other words, it only promises to provide itself. If the module
	 * actually fulfills additional contracts (eg, providing implementations of generic filesystem services), it should
	 * override this method.
	 *
	 * This method is used to determine which order setup and shutdown is called on modules.
	 *
	 * @return the contracts this module satisfies.
	 */
	Module::ServiceList Module::getProvides() const {
		return {getName()};
	}

	/**
	 * Perform any setup required for this Module.
	 *
	 * This is a stub, to avoid modules that don't do anything fancy having to implement a blank function.
	 *
	 * This, this should be RAII, no, I'm not going to fix it right now. Trying to fix it was causing more headaches than
	 * it was solving...
	 */
	void Module::setup(Generator*) {

	}

	/**
	 * Perform any teardown required for this Module.
	 *
	 * This is a stub, to avoid modules that don't do anything fancy having to implement a blank function.
	 *
	 * This, this should be RAII, no, I'm not going to fix it right now. Trying to fix it was causing more headaches than
	 * it was solving...
	 */
	void Module::teardown() {

	}

} // namespace fggl:services
